/*
 * $Id: ChannelState.java 292 2006-02-28 23:30:47Z davidb $
 *
 * Copyright (c) 2005 VeriSign, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package se.rfc.unbound.server;

import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.util.LinkedList;

import se.rfc.unbound.DNSEvent;

/**
 * This is a per-channel state object. This tracks various per-channel.
 * attributes.
 * 
 * @author davidb
 * @version $Revision: 292 $
 */
public class ChannelState
{

  /**
   * If isPermanent is true, then the channel this is attached to will never
   * be closed.
   */
  private boolean    mIsPermanent;
  /**
   * If isOutbound (and not isPermanent) then the attached channel will be
   * closed after a READ operation. If false, after a WRITE operation.
   */
  private boolean    mIsOutbound;

  /**
   * A queue of write events. Events are pulled off the queue to be written.
   * If the write did not complete, the are pushed back onto the queue. The
   * intent is that WriteEvents are handled on a first-come-first-serve basis.
   */
  private LinkedList mWriteEventQueue;

  /** For TCP read operations. */
  private ByteBuffer mReadData;

  /**
   * Create a new channel state object.
   * 
   * @param isPermanent if true, then this channel will be considered
   *          permanent and only closed when the server exits/restarts.
   * @param isOutbound if true, then the channel is used for sending queries
   *          and receiving responses, false means that the channel is used
   *          for receiving request and sending responses. This only is
   *          relevent if the channel is not permanent.
   */
  public ChannelState(boolean isPermanent, boolean isOutbound)
  {
    setPermanent(isPermanent);
    setOutbound(isOutbound);
  }

  /**
   * @return The current read buffer.
   */
  public ByteBuffer getReadBuffer()
  {
    return mReadData;
  }

  /**
   * Set the current read buffer.
   * 
   * @param buffer The buffer.
   */
  public void setReadBuffer(ByteBuffer buffer)
  {
    mReadData = buffer;
  }

  /**
   * @return true if the channel is used for sending requests and receiving
   *         queries (and not the opposite).
   */
  public boolean isOutbound()
  {
    return mIsOutbound;
  }

  /**
   * @return true if the channel is permanent.
   */
  public boolean isPermanent()
  {
    return mIsPermanent;
  }

  /**
   * Set the outbound status for this channel
   * 
   * @param isOutbound true if this channel is used to send requests.
   */
  public void setOutbound(boolean isOutbound)
  {
    mIsOutbound = isOutbound;
  }

  /**
   * Set the permanent status for this channel.
   * 
   * @param isPermanent The value.
   */
  public void setPermanent(boolean isPermanent)
  {
    mIsPermanent = isPermanent;
  }

  /**
   * Create and add a write event to this channel's write queue.
   * 
   * @param data The actual data to write.
   * @param target The IP address and port to send the data to.
   * @param event The event generating this write actaion.
   * @param matchState The match state that goes with this event.
   */
  public void addWriteEvent(byte[] data, InetSocketAddress target,
      DNSEvent event, Object matchState)
  {
    WriteEvent we = new WriteEvent(data, target, event, matchState);
    if (mWriteEventQueue == null)
    {
      mWriteEventQueue = new LinkedList();
    }
    mWriteEventQueue.add(we);
  }

  /**
   * @return The next write event in the queue, null if there are no more
   *         write events.
   */
  public WriteEvent nextWriteEvent()
  {
    if (mWriteEventQueue == null) return null;
    if (mWriteEventQueue.size() == 0) return null;
    return (WriteEvent) mWriteEventQueue.removeFirst();
  }

  /**
   * Push an unfinished write event back onto the queue.
   * 
   * @param write_event The unfinished write event.
   */
  public void unfinishedWriteEvent(WriteEvent write_event)
  {
    mWriteEventQueue.add(write_event);
  }

  /**
   * @return true if there are any write events in this channel's queue.
   */
  public boolean hasWriteEvents()
  {
    return !mWriteEventQueue.isEmpty();
  }
}
